package pizza.config;
// System configuration based on existing Hibernate configuration
// for a single DB.
// client-server: Hibernate config has user/pw for a JDBC connection
// web app: Hibernate config has JNDI name to look up a Datasource 
//  in the app server's JNDI repository, where the app server (tomcat
// for example) configuration has the user/pw for a JDBC connection.
// Either way, Hibernate can create a Hibernate Session for the app
// based on its config.  Here we set up singleton service objects
// and DAOs for the app.

import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;

import pizza.persistence.HibernateUtil;
import pizza.dao.DbDAO;
import pizza.dao.AdminDAO;
import pizza.dao.PizzaOrderDAO;
import pizza.service.AdminService;
import pizza.service.StudentService;

public class PizzaSystemConfig {
	// for ease of testing, handle only a few rooms--
	public static final int NUM_OF_ROOMS = 10;

	// log for this class (part of app log)
	private static Log log = LogFactory.getLog(PizzaSystemConfig.class);

	// the service objects in use, representing all lower layers to the app
	private static AdminService adminService;

	private static StudentService studentService;

	// for in-server case only, name like "jdbc/mydbname"
	private static String jndiName = null;

	// For a web app, a DataSource is held in the JNDI repository of the app
	// server, configured there for this app, for ex., in conf/context.xml for tomcat.
	// Hibernate looks it up based on the properties in hibernate.cfg.xml
	// for ex., for HSQL given JNDI name jdbc/hsql, long name
	// java:comp/env/jdbc/hsql,--
	// <property name="hibernate.connection.datasource">java:comp/env/jdbc/hsql</property>
	// <property name="hibernate.dialect">org.hibernate.dialect.HSQLDialect</property>
	// Here we query the hibernate config to find the JNDI name, and if it's
	// not there, assume the app is not a web app
	
	static {
		try {
			jndiName = HibernateUtil.getJndiName();
			if (jndiName == null) {
				log.info("No JNDI name available: if this is a web app, hibernate config needs work");
			} else {
				// web app with promising jndiName to work from
				log.info("JNDI name from hibernate: " + jndiName);
				configureServices();
			}
		} catch (Exception e) {
			reportException(log, e);
		}
	}

	// Note no shutdown code: everything gets cleaned up when the client-server
	// program exits, and the DataSource connections are released at the ends of
	// the various transactions

	// configure service objects--called explicitly for client-server
	// or from above static initializer for web app 
	// The service objects get what they need at creation-time
	// This is constructor injection (pg. 9 of Johnson et al)
	//	 Note that the isolation level is part of the JNDI config 
	//  of the Datasource in the web app case.
	public static void configureServices() {
		try {
			String hibernateDialect = HibernateUtil.getHibernateDialect();
			log.info("got hibernate.dialect from hibernate: "
					+ hibernateDialect);
			log.info("Trying test connection to database, getting isolation level...");
			log.info("DB is at isolation level (1=RU, 2=RC, 4=RR, 8=ser.) "
					+ HibernateUtil.getIsolationLevel());
			log.info("calling dbDAO constructor");
			DbDAO dbDAO = new DbDAO();
			// configure rest of service and DAO singleton objects--
			// These objects get what they need at creation-time
			// This is "constructor injection"
			AdminDAO adminDAO = new AdminDAO(dbDAO);
			PizzaOrderDAO pizzaOrderDAO = new PizzaOrderDAO(dbDAO);
			adminService = new AdminService(dbDAO, adminDAO, pizzaOrderDAO);
			studentService = new StudentService(dbDAO, pizzaOrderDAO, adminDAO);
		} catch (Exception e) {
			reportException(log, e);
		}
	}

	// Report an exception to the application log--
	// and return the logMsg for callers to use
	public static String reportException(Log log, Exception e) {
		String logMsg = e.getMessage(); 
		if (e.getCause() != null)
			logMsg += "cause: " + e.getCause().getMessage();
		if (e.getCause().getCause() != null)
			logMsg += "inner cause: " + e.getCause().getCause().getMessage();
		log.error(logMsg);
		return logMsg;
	}

	// Let the apps get the business logic layer services
	public static AdminService getAdminService() {
		return adminService;
	}

	public static StudentService getStudentService() {
		return studentService;
	}
}
